@@ -2,7 +2,7 @@ package Test2::Harness::Job::Runner::IPC;
 use strict;
 use warnings;
 
-our $VERSION = '0.001074';
+our $VERSION = '0.001075';
 
 use Test2::Harness::Util qw/open_file write_file local_env/;
 use Test2::Harness::Util::IPC qw/run_cmd/;
@@ -29,7 +29,7 @@ sub command_file {
 
 sub command {
     my $class = shift;
-    my ($test, $event_file, $inc) = @_;
+    my ($test, $event_dir, $inc) = @_;
 
     my $job = $test->job;
 
@@ -42,7 +42,7 @@ sub command {
         $job->mem_usage ? ('-MTest2::Plugin::MemUsage') : (),
         (map { "-M$_" } @{$job->load_import || []}),
         (map { "-m$_" } @{$job->load        || []}),
-        $job->use_stream ? ("-MTest2::Formatter::Stream=file,$event_file") : (),
+        $job->use_stream ? ("-MTest2::Formatter::Stream=dir,$event_dir") : (),
         $job->times ? ('-MTest2::Plugin::Times') : (),
         $class->command_file($test),
         @{$job->args},
@@ -55,7 +55,7 @@ sub run {
 
     my $job = $test->job;
 
-    my ($in_file, $out_file, $err_file, $event_file) = $test->output_filenames;
+    my ($in_file, $out_file, $err_file, $event_dir) = $test->output_filenames;
 
     my $out_fh = open_file($out_file, '>');
     my $err_fh = open_file($err_file, '>');
@@ -67,7 +67,7 @@ sub run {
         %{$job->env_vars},
         $job->use_stream
             ? ( T2_FORMATTER => 'Stream',
-                T2_STREAM_FILE => $event_file )
+                T2_STREAM_DIR => $event_dir )
             : (),
     };
 
@@ -78,7 +78,7 @@ sub run {
     local_env $env => sub {
         $pid = run_cmd(
             chdir   => $job->ch_dir,
-            command => sub { $class->command($test, $event_file, \@inc) },
+            command => sub { $class->command($test, $event_dir, \@inc) },
             stdin   => $in_fh,
             stdout  => $out_fh,
             stderr  => $err_fh,
