@@ -2,7 +2,7 @@ package Test2::Harness::Job::Runner::Open3;
 use strict;
 use warnings;
 
-our $VERSION = '0.001074';
+our $VERSION = '0.001075';
 
 use IPC::Open3 qw/open3/;
 use Test2::Harness::Util qw/open_file write_file local_env/;
@@ -30,7 +30,7 @@ sub command_file {
 
 sub command {
     my $class = shift;
-    my ($test, $event_file, $inc) = @_;
+    my ($test, $event_dir, $inc) = @_;
 
     my $job = $test->job;
     my %seen;
@@ -43,7 +43,7 @@ sub command {
         $job->mem_usage ? ('-MTest2::Plugin::MemUsage') : (),
         (map {"-M$_"} @{$job->load_import || []}),
         (map {"-m$_"} @{$job->load || []}),
-        $job->use_stream ? ("-MTest2::Formatter::Stream=file,$event_file") : (),
+        $job->use_stream ? ("-MTest2::Formatter::Stream=dir,$event_dir") : (),
         $job->times ? ('-MTest2::Plugin::Times') : (),
         $class->command_file($test),
         @{$job->args},
@@ -56,7 +56,7 @@ sub run {
 
     my $job = $test->job;
 
-    my ($in_file, $out_file, $err_file, $event_file) = $test->output_filenames;
+    my ($in_file, $out_file, $err_file, $event_dir) = $test->output_filenames;
 
     my $out_fh = open_file($out_file, '>');
     my $err_fh = open_file($err_file, '>');
@@ -81,7 +81,7 @@ sub run {
             chdir($dir) or die "Could not chdir: $!";
         }
 
-        my @cmd = $class->command($test, $event_file, \@inc);
+        my @cmd = $class->command($test, $event_dir, \@inc);
         $pid = open3(
             '<&' . fileno($in_fh), ">&" . fileno($out_fh), ">&" . fileno($err_fh),
             @cmd,
